<?php

/*
 * Plugin Name: Recapture for WooCommerce
 * Plugin URI: https://recapture.io/
 * Description: Recapture helps you increase revenue by automatically recovering abandoned carts.
 * Version: 1.0.5
 * Author: Recapture
 * Text Domain: recapture
 *
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

if (!class_exists('Recapture')) {

    include(dirname(__FILE__) . '/class-recapture-utils.php');
    include(dirname(__FILE__) . '/platforms/class-base-platform.php');
    include(dirname(__FILE__) . '/platforms/class-edd.php');
    include(dirname(__FILE__) . '/platforms/class-woocommerce.php');

    define('RECAPTURE_VERSION', '1.0.5');
    define('RECAPTURE_TEXT_DOMAIN', 'recapture');
    define('RECAPTURE_TARGET_PLATFORM', 'WooCommerce');

    class Recapture
    {
        private $platform = null;

        /**
         * Constructor for the plugin.
         *
         * @access        public
         */
        public function __construct()
        {
            add_action('init', [$this, 'load_plugin_textdomain']);

            if (RecaptureWooCommerce::is_ready()) {
                $this->platform = new RecaptureWooCommerce();
            } else {
                add_action('admin_init', [$this, 'recapture_nag_ignore']);
                add_action('admin_notices', [$this, 'plugin_missing_notice']);
                return;
            }

            // Add the plugin page Settings and Docs links
            add_filter('plugin_action_links_' . plugin_basename(__FILE__),
                [$this, 'recapture_plugin_links']);

            // standard wordpress actions
            add_action('init', [$this, 'check_version']);
            add_action('init', [$this, 'finish_recapture_connection']);
            add_action('admin_enqueue_scripts', [$this, 'admin_enqueue_scripts']);
            add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);

            // set the cart id, overwrite the current cart if any
            add_action('init', [$this, 'set_cart_id_from_Url']);
            add_action('wp_loaded', [$this, 'regenerate_cart_from_url']);
            add_action('admin_notices', [$this, 'admin_notices']);
            add_action('admin_menu', [$this, 'admin_page']);
            add_action('wp_footer', [$this, 'add_footer_scripts']);

            // ajax handlers
            add_action('wp_ajax_recapture_connection_status', [$this, 'ajax_is_connected']);

            // Recapture connection actions
            add_action('admin_post_recapture_disconnect',
                [$this, 'disconnect_recapture']);

            add_action('admin_post_recapture_connect',
                [$this, 'connect_recapture']);

            add_action('admin_post_recapture_confirm_disconnect',
                [$this, 'confirm_disconnect_recapture']);

            // setup the actions for this provider;
            $this->platform->add_actions();
        }

        function load_plugin_textdomain() {
            $location = basename(dirname(__FILE__)).'/languages/';
            load_plugin_textdomain(RECAPTURE_TEXT_DOMAIN, FALSE, $location);
        }

        function enqueue_scripts() {
            $this->platform->enqueue_scripts();
        }

        public function regenerate_cart_from_url()
        {
            $req = (object)$_GET;

            if (!isset($req->racart) || !isset($req->contents)) {
                return;
            }

            $this->platform->regenerate_cart_from_url($req->racart, $req->contents);
        }

        function admin_enqueue_scripts() {
            // add our style
            wp_register_style(
                'recapture_styles',
                plugins_url('/css/styles.css', __FILE__),
                false,
                RECAPTURE_VERSION,
                'all'
            );

            wp_enqueue_style('recapture_styles');

            // Add script
            wp_register_script(
                'recapture_script',
                plugins_url('/js/admin.js', __FILE__),
                RECAPTURE_VERSION,
                'all');

            wp_localize_script(
                'recapture_script',
                '___recapture',
                ['ajax' => admin_url('admin-ajax.php')]
            );

            wp_enqueue_script('recapture_script');
        }

        public function set_cart_id_from_Url()
        {
            $req = (object)$_GET;

            if (isset($req->racart)) {
                RecaptureUtils::set_cart_id($req->racart);
            } else {
                RecaptureUtils::set_cart_id_if_missing();
            }
        }

        public function ajax_is_connected() {
            wp_send_json_success(RecaptureUtils::is_connected());
        }

        /**
         * Set up admin notices
         *
         * @access        public
         * @return        void
         */
        public function admin_notices()
        {
            $screen = get_current_screen();

            // If the Merchant ID field is empty
            if (get_option('recapture_api_key') || $screen->base == 'toplevel_page_recapture') {
                return;
            }

            ?>
            <div class="updated">
                <p>
                    <?php _e('Please connect website to Recapture', RECAPTURE_TEXT_DOMAIN); ?>
                    <a href="<?= admin_url('admin.php?page=recapture') ?>">
                        <?php _e('here', RECAPTURE_TEXT_DOMAIN); ?>
                    </a>
                </p>
            </div>
            <?php
        }

        /**
         * Initialize the Recapture menu
         *
         * @access        public
         * @return        void
         */
        public function admin_page()
        {
            add_menu_page(
                'Recapture',
                'Recapture',
                'manage_options',
                'recapture',
                [$this, 'admin_options'],
                plugins_url('images/recapture.png', __FILE__),
                58);

            add_submenu_page(
                null,
                'Disconnect Recapture',
                'Disconnect Recapture',
                'manage_options',
                'recapture-confirm-disconnect',
                [$this, 'confirm_disconnect_recapture']
            );

            add_action('admin_init', [$this, 'register_settings']);
        }

        /**
         * Register settings for Recapture
         *
         * @access        public
         * @return        void
         */
        public function register_settings()
        {
            register_setting('recapture-settings-group',
                'recapture_api_key');

            register_setting('recapture-settings-group',
                'recapture_custom_loader_url');

            register_setting('recapture-settings-group',
                'recapture_custom_recapture_host');
        }

        /**
         * Add options to the Recapture menu
         *
         * @access        public
         * @return        void
         */
        public function admin_options()
        {
            $discount = RecaptureUtils::get_discount_details();

            ?>
            <div class="wrap">
                <h2>Recapture</h2>

                <form method="post" action="<?php echo admin_url('admin-post.php'); ?>" target="_blank">
                    <?php if (RecaptureUtils::is_connected()) { ?>
                        <p>
                            <?= __('Recapture has been successfully connected.', RECAPTURE_TEXT_DOMAIN) ?>
                        </p>
                        <p>
                            <a href="https://recapture.io/account/abandoned-carts" target="_blank">
                                Campaign Analytics
                            </a>
                            &nbsp;|&nbsp;
                            <a href="https://recapture.io/account/abandoned-carts/campaigns" target="_blank">
                                Manage Campaigns
                            </a>
                            &nbsp;|&nbsp;
                            <a href="https://docs.recapture.io/" target="_blank">
                                Docs and Support
                            </a>
                        </p>
                        <h3 class="recapture-disconnect">Disconnect this site</h3>
                        <p>
                            <a href="<?= admin_url('admin.php?page=recapture-confirm-disconnect') ?>" class="button">
                                <?php _e('Disconnect from Recapture', RECAPTURE_TEXT_DOMAIN) ?>
                            </a>
                        </p>

                    <?php } else { ?>
                        <?php
                        if ($discount) {
                        ?>
                            <div class="recapture-discount">
                                <?= $discount->description ?>
                                <?php submit_button(__('Get Started', RECAPTURE_TEXT_DOMAIN), 'large', 'submit', false); ?>
                            </div>
                        <?php
                        }
                        ?>
                        <h2>Welcome to Recapture!</h2>
                        <p>
                            The best way to recover abandoned carts for <?php echo RECAPTURE_TARGET_PLATFORM ?>
                        </p>
                        <p>
                            To start increasing your store's revenue, you'll need to connect your site to <a href="https://recapture.io">Recapture</a>
                        </p>
                        <p>
                            <input type="hidden" name="action" value="recapture_connect">
                            <?php submit_button(__('Connect to Recapture', RECAPTURE_TEXT_DOMAIN), 'primary', 'submit', false); ?>
                        </p>
                        <p>
                            If you're having issues connecting, don't hesitate to <a href="https://recapture.io">contact us</a>
                        </p>
                        <script>
                        window.RecaptureAdmin.monitorAccountConnection();
                        </script>
                    <?php } ?>
                </form>
            </div>
            <?php
        }

        function connect_recapture()
        {
            // redirect to the connection url
            $url = RecaptureUtils::get_connect_url(
                $this->platform->get_name(),
                admin_url('admin.php?page=recapture'),
                admin_url('admin.php?page=recapture')
            );
            wp_redirect($url);
            exit();
        }

        function disconnect_recapture()
        {
            RecaptureUtils::delete_api_key();
            wp_redirect(admin_url('admin.php?page=recapture'));
            exit();
        }

        function confirm_disconnect_recapture()
        {
            ?>
            <div class="wrap">
                <h2>Recapture</h2>
                <form method="post" action="<?php echo admin_url('admin-post.php'); ?>">
                    <p>
                        <?php _e('Disconnecting will stop new carts from being added and orders from being tracked', RECAPTURE_TEXT_DOMAIN) ?>
                        <input type="hidden" name="action" value="recapture_disconnect">
                    <p>
                        <?php submit_button(__('I understand, disconnect my store', RECAPTURE_TEXT_DOMAIN), 'primary', 'submit', false); ?>
                        &nbsp;
                        <a href="<?= admin_url('admin.php?page=recapture') ?>" class="button">
                            <?php _e('Cancel', RECAPTURE_TEXT_DOMAIN) ?>
                        </a>
                    </p>
                </form>
            </div>
            <?php
        }

        function finish_recapture_connection()
        {
            if (!empty($_GET['connect_recapture']) &&
                !empty($_GET['api_key']) &&
                !empty($_GET['token'])  &&
                $_GET['token'] == RecaptureUtils::get_authenticator_token()
            ) {
                RecaptureUtils::set_api_key(sanitize_text_field($_GET['api_key']));

                // clear any discount code??
                RecaptureUtils::clear_discount_details();
                die('Connected!');
            }
        }

        function check_version()
        {
            if (isset($_GET['get_recapture_version'])) {
                $res = [
                    'version' => RECAPTURE_VERSION,
                    'platform' => $this->platform->get_name()
                ];
                echo json_encode($res);
                die();
            }
        }

        /**
         * Add scripts to the footer of th checkout or thank you page
         *
         * @access        public
         * @return        void
         */
        public function add_footer_scripts()
        {
            $api_key = RecaptureUtils::get_api_key();

            if (!$api_key || empty($api_key)) {
                return;
            }

            $cache_bust = round((time() / (60 * 10)));
            $url = RecaptureUtils::get_loader_url() . '?v=' . $cache_bust;

            ?>
            <script type='text/javascript' src="<?= $url ?>"></script>
            <script type='text/javascript'>
              ra('init', ['<?= $api_key ?>']);
              <?php if ($this->platform->is_product_page()) { ?>
              ra('product', [{}]);
              <?php } else { ?>
              ra('page');
              <?php } ?>
              ra('email');
            </script>
            <?php
        }

        /**
         * Plugin page links
         *
         * @param array $links
         * @return array
         */
        function recapture_plugin_links($links)
        {
            $links['settings'] = '<a href="' . admin_url('admin.php?page=recapture&settings-updated=true') . '">' . __('Settings', 'Recapture') . '</a>';
            return $links;
        }


        /**
         * Easy Digital Downloads plugin missing notice.
         *
         * @return string
         */
        public function plugin_missing_notice()
        {
            global $current_user;
            $user_id = $current_user->ID;
            if (!get_user_meta($user_id, 'recapture_missing_plugin_nag')) {
                $message = sprintf(
                    __('Recapture needs %s to be installed and active.', RECAPTURE_TEXT_DOMAIN),
                    RECAPTURE_TARGET_PLATFORM);

                $hide_message = __('Hide Notice', RECAPTURE_TEXT_DOMAIN);

                echo "
                    <div class=\"error\">
                        <p>
                            $message
                            <a href=\"?recapture_missing_plugin_nag=0\">$hide_message</a>
                        </p>
                    </div> 
                ";
            }

            return null;
        }


        /**
         *  Remove the nag if user chooses
         */
        function recapture_nag_ignore()
        {
            global $current_user;
            $user_id = $current_user->ID;

            if (isset($_GET['recapture_missing_plugin_nag']) &&
                $_GET['recapture_missing_plugin_nag'] == '0') {
                add_user_meta($user_id, 'recapture_missing_plugin_nag', 'true', true);
            }
        }
    }

    function recapture_plugins_loaded()
    {
        new Recapture();
    }
    add_action('plugins_loaded', 'recapture_plugins_loaded');

    function recapture_plugin_activated($plugin) {
        $is_ready = RecaptureWooCommerce::is_ready();

        if ($plugin == plugin_basename(__FILE__) && !RecaptureUtils::is_connected() && $is_ready) {
            wp_redirect(admin_url('admin.php?page=recapture'));
            exit();
        }
    }
    add_action('activated_plugin', 'recapture_plugin_activated');

} // End if class_exists check.